# -*- mode: python -*-
# -*- coding: utf-8 -*-

##############################################################################
#
# Gestion scolarite IUT
#
# Copyright (c) 1999 - 2023 Emmanuel Viennet.  All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Emmanuel Viennet      emmanuel.viennet@viennet.net
#
##############################################################################

"""Feuille excel pour préparation des jurys classiques (non BUT)
"""
import collections
import time

from flask import flash, abort

from app.but import jury_but
from app.but.cursus_but import EtudCursusBUT
from app.but.jury_but import DecisionsProposeesAnnee
from app.but.prepajury_desc import ParcoursDesc, liste_annees, FormSemestreDesc
from app.comp import res_sem
from app.comp.res_but import ResultatsSemestreBUT
from app.models import (
    FormSemestre,
    Identite,
    ScolarAutorisationInscription,
    ApcParcours,
    Formation,
    ApcCompetence,
)
import app.scodoc.sco_utils as scu
from app.but.prepajury_xl import ScoExcelBook


class Element:
    def __init__(self):
        self.note = None
        self.resultat = None
        self.format = 0

    def set_note(self, note):
        self.note = note

    def set_res(self, res):
        self.res = res

    def get_res(self):
        return self.res

    def get_note(self):
        return self.note


class ElementUE(Element):
    def __init__(self):
        super().__init__()


class ElementNiveau(Element):
    def __init__(self, etudiant):
        super().__init__()
        self.etudiant = etudiant

    def add_ue_status(self, semestre_idx, status):
        ue_id = status["ue"]["id"]
        self.ues[semestre_idx].append(status)
        self.etudiant.ues[ue_id] = status


class ElementAnnee(Element):
    def __init__(self, etudiant, annee_desc):
        super().__init__()
        self.etudiant = etudiant
        self.anneeDesc = annee_desc
        self.niveaux = {}


    def getNiveau(self, niveau_id):
        return self.niveaux[niveau_id]


class ElementFormsemestre(Element):
    def __init__(self, formsemestre_id=None, formsemestre=None):
        super().__init__()
        if formsemestre is None:
            formsemestre = FormSemestre.get_formsemestre(formsemestre_id)
        self.formsemestre = formsemestre
        self.deca = None

    def get_deca(self):
        if self.deca is None:
            self.deca = 0
        return self.deca


class ListeElements:
    def __init__(self):
        self.elements = []

    def add(self, element):
        self.elements.append(element)

    def last(self):
        return self.elements[-1]


class EtudiantJury:
    def __init__(self, ident, formation, current_formsemestre, contexte):
        self.contexte = contexte
        self.ident = ident
        self.formsemestre = current_formsemestre
        self.formsemestres = []
        self.formation = formation
        self.cursus: EtudCursusBUT = EtudCursusBUT(ident, formation)
        self.parcour = self.cursus.inscriptions[-1].parcour
        # Variables calculées
        self.history = []
        #
        self.formsemestre_par_semestre_idx = collections.defaultdict(lambda: ListeElements())
        self.niveau_par_annee_et_competence : ListeElements = None
        self.ues_par_annee_et_competence : list(ListeElements) = {}
        #
        for inscription in sorted(
                self.cursus.inscriptions, key=lambda x: x.formsemestre.date_debut
        ):
            formsemestre = inscription.formsemestre
            element_formsemestre = contexte.get_element_formsemestre(formsemestre.formsemestre_id)
            Sx = f"S{formsemestre.semestre_id}"
            self.formsemestre_par_semestre_idx[formsemestre.semestre_id].add(formsemestre)
            etat = inscription.etat
            if etat != "I":
                Sx += " (Dem)" if etat == "D" else f" ({etat})"
            self.history.append(Sx)

    def select(self, annee_idx=None, periode_idx=None, competence_id=None):
        pass

    def fill_in(self, parcourDescr, semestres):
        # Remplissage des competences
        self.competences = self.cursus.competences
        # Remplissage des formsemestres
        for formsemestre_id in self.formsemestres:
            formsemestre_desc = self.contexte.get_element_formsemestre(formsemestre_id)
            semestre_idx = formsemestre_desc.formsemestre.semestre_id
            self.formsemestre_par_semestre_idx[semestre_idx] = formsemestre_desc
        for semestre_idx in range(1, 6):
            if semestre_idx in self.formsemestre_par_semestre_idx:
                formsemestre_desc = self.formsemestre_par_semestre_idx[semestre_idx]
                resultats = formsemestre_desc.get_resultats()
                ues = resultats.etud_ues(self.ident.etudid)
                for ue in ues:
                    competence_id = ue.niveau_competence.competence_id
                    if competence_id in self.competences:
                        status = resultats.get_etud_ue_status(self.ident.etudid, ue.id)
                        element_ue = ElementUE()
                        element_ue.set_note(status.moy)
                        self.ues_par_annee_et_competence[(semestre_idx, competence_id)] = element_ue
        for competence_id, val_1 in self.cursus.validation_par_competence_et_annee.items():
            for annee, validation in self.cursus.validation_par_competence_et_annee[competence_id].items():
                element_niveau = ElementNiveau(self)
                self.niveau_par_annee_et_competence[annee][competence_id] = element_niveau
                element_niveau.set_res(validation.code)
        pass

            # deca = DecisionsProposeesAnnee(self.ident, formsemestre)
        # Remplissage des niveaux de compétence
        # for (competence_id, item) in self.cursus.validation_par_competence_et_annee.items():
        #     for (annee, validation) in item.items():
        #         self.niveau_par_annee_and_competence[(annee, competence_id)].res = validation.code
        # for annee, idx in [("BUT1", 1), ("BUT2", 2), ("BUT3", 3)]:
        #     formsemestre = self.formsemestre_par_semestre_idx.get(idx + 1, self.formsemestre_par_semestre_idx.get(idx, None))
        #     if formsemestre is not None:
        #         deca = DecisionsProposeesAnnee(self.ident, formsemestre)

        # for (competence_id, competenceDesc) in parcourDescr.competences.items():
        #     self.competences[competence_id] = CompetenceEtudiantJury(
        #         self, competenceDesc, semestres
        #     )
        # self.annees = {
        #     codeAnnee: _Annee(self, anneeDesc)
        #     for codeAnnee, anneeDesc in parcourDescr.annees.items()
        # }
        # for semestre, annee in [(2, "BUT1"), (4, "BUT2"), (6, "BUT3")]:
        #     if semestre in self.formsemestre_par_semestre_idx:
        #         self.deca[annee] = jury_but.DecisionsProposeesAnnee(
        #             self.ident, self.formsemestre_par_semestre_idx[semestre]
        #         )
        #     elif semestre - 1 in self.formsemestre_par_semestre_idx:
        #         self.deca[annee] = jury_but.DecisionsProposeesAnnee(
        #             self.ident, self.formsemestre_par_semestre_idx[semestre - 1]
        #         )
        #     if annee in self.deca:
        #         for rcue in self.deca[annee].rcues_annee:
        #             ue_id1 = rcue.ue_1.id
        #             self.notes_ues[ue_id1] = rcue.moy_ue_1
        #             ue_id2 = rcue.ue_2.id
        #             self.notes_ues[ue_id2] = rcue.moy_ue_2
        #             rcue_id = rcue.ue_1.niveau_competence_id
        #             self.notes_rcues[rcue_id] = rcue.moy_rcue

    def add_formsemestre(self, formsemestre_id):
        self.formsemestres.append(formsemestre_id)

    def getDipl(self, codeAnnée):
        return ""

    def getNote(self, codeAnnée, niveau_id=None, ue_id=None):
        annee = self.annees.get(codeAnnée, None)
        if annee is None:
            return "-"
        if niveau_id is None:
            return ""
        elif ue_id is None:
            return self.notes_rcues.get(niveau_id, "")
        return self.notes_ues.get(ue_id, "")

    def getRes(self, codeAnnée, niveau_id=None, ue_id=None):
        if niveau_id is None:
            return ""
        elif ue_id is None:
            return ""
        return ""

    def getData(self):
        result = [
            self.ident.id,
            self.ident.code_nip,
            self.ident.civilite,
            self.ident.nom,
            self.ident.prenom,
            self.ident.etat_civil_pv(with_paragraph=False),
            self.parcour.code,
            ", ".join(self.history),
        ]
        return result


class _Compilation:
    def __init__(self, formsemestre: FormSemestre):
        # Dictionnaire des éléments
        self.inv_formsemestres = {}
        # self.inv_niveaux = {}
        # self.inv_ues = {}
        # self.inv_annees = {}
        self.inv_parcours = {}
        #
        formsemestre_id = formsemestre.formsemestre_id
        self.formation: Formation = formsemestre.formation
        self.consider_formsemestre(formsemestre=formsemestre)
        formsemestre_desc = self.get_element_formsemestre(formsemestre_id)
        for ident in formsemestre_desc.get_resultats().get_inscrits(order_by="moy"):
            etudiant: EtudiantJury = EtudiantJury(ident, self.formation, formsemestre, self)
            # Ajout de tous les semestres de l'étudiant
            for inscription in sorted(etudiant.cursus.inscriptions, key=lambda x: x.formsemestre.date_debut):
                formsemestre = inscription.formsemestre
                if (
                        formsemestre.formation.referentiel_competence
                        == self.formation.referentiel_competence
                ):
                    self.consider_formsemestre(formsemestre=formsemestre)
                    etudiant.add_formsemestre(formsemestre.formsemestre_id)
            # Prise en compte du parcours
            scodocParcour = etudiant.parcour
            if scodocParcour is None:
                parcourCode = "TC"
            else:
                parcourCode = scodocParcour.code
            if parcourCode in self.inv_parcours:
                parcoursDesc = self.inv_parcours[parcourCode]
            else:
                parcoursDesc = ParcoursDesc(self.formation, scodocParcour)
                self.inv_parcours[parcourCode] = parcoursDesc
            parcoursDesc.add_etudiant(etudiant)
            etudiant.fill_in(parcoursDesc, self)

    def get_element_formsemestre(self, formsemestre_id):
        return self.inv_formsemestres.get(formsemestre_id, None)

    def consider_formsemestre(self, formsemestre: FormSemestre):
        formsemestre_id = formsemestre.formsemestre_id
        if formsemestre_id not in self.inv_formsemestres:
            formsemestre_desc = FormSemestreDesc(formsemestre)
            self.inv_formsemestres[formsemestre_id] = formsemestre_desc

    def add_semestre(self, formsemestre_id, semestre: ElementFormsemestre = None):
        if not formsemestre_id in self.ElemFormSemestres:
            if semestre is None:
                semestre = ElementFormsemestre(formsemestre_id)
            self.ElemFormSemestres[formsemestre_id] = ElementFormsemestre(formsemestre_id)

    def computes_decision(self):
        pass

    def make_excel(self, filename: str):
        workbook = ScoExcelBook()
        for parcoursCode, parcours in self.inv_parcours.items():
            parcours.generate(workbook)

        mime, suffix = scu.get_mime_suffix("xlsx")
        xls = workbook.generate()
        return scu.send_file(xls, filename=filename, mime=mime, suffix=suffix)


def feuille_preparation_jury_but(formsemestre_id):
    if not isinstance(formsemestre_id, int):
        abort(404)
    formsemestre = FormSemestre.get_formsemestre(formsemestre_id)
    # res: NotesTableCompat = res_sem.load_formsemestre_results(formsemestre)
    """Feuille excel pour préparation des jurys adaptée pour le BUT."""
    breakpoint()
    compilation = _Compilation(formsemestre)
    compilation.computes_decision()
    filename = scu.sanitize_filename(
        f"""{'jury'}-{formsemestre.titre_num()}-{time.strftime("%Y-%m-%d")}"""
    )
    return compilation.make_excel(filename)
